"use strict";
this.name = "Cabal_Common_Functions";
this.author = "Cmd.Cheyd, PhantorGorth and Svengali";
this.copyright = "(C)2010-2013, License:CC-by";
this.description = "Libraryscript";
this.version = "1.7";

/* class: Cabal_Common
This is the main class for the helper library with its members. Scripts can instantiate a copy, e.g.
> this.myHelper = new worldScripts.Cabal_Common_Functions.Cabal_Common();
*/
this.Cabal_Common = function(){}
Cabal_Common.prototype = {
	constructor: Cabal_Common,
	// Property to give OXPs a chance to check the required lib min. version easily.
	internalVersion: 15,
	// v1.77 spacer
	spacer: String.fromCharCode(31),
	spacerWidth: defaultFont.measureString(String.fromCharCode(31)),
	spacerStandard: String.fromCharCode(32),
	spacerWidthStandard: defaultFont.measureString(String.fromCharCode(32)),
	/* method: baseChange(). Changes the base of n.
	Parameters:
		n - Number. To be changed.
		to - Number. Base which should be used to convert to. Usually 2,10 or 16.
		from - Number. Optional. Base from which should be converted. Usually 2,10 or 16.
	Returns:
		str - String. Base changed.
	Author: Dr.J R Stockton.
	*/
	baseChange: function(n,to,from){
		return(parseInt(n,from || 10).toString(to));
	},
	/* method: clamp()
	Clamps value to max...min.
	Paramaters:
		value - Number. To be clamped.
		max - Number. Maximum.
		min - Number. Minimum.
	Returns:
		n - Number. Clamped.
	*/
	clamp: function(value,max,min){
		return(((value>max)?max:(value<min)?min:value));
	},
	/* method: msbPos()
	Returns position of most significant bit of n.
	Paramaters:
		n - Number. To be evaluated.
	Returns:
		n - Number. The position of the most significant bit or 0 (zero).
	Author: Dr.J R Stockton.
	*/
	msbPos: function(n){
		var e = 15;
		for(var r=8;r>0;r>>=1){
			e = (n<1<<e)?e-r:e+r;
		}
		return((n<1<<e)?e:e+1);
	},
	/* method: nBitsUsed()
	Returns number of bits set in n. If n<0 a bitwise NOT will be applied before execution.
	Parameters:
		n - Number. To be evaluated.
	Returns:
		e - Number. The counted number of bits set to 1.
	*/
	nBitsUsed: function(n){
		var e = 0;
		if(n<0) n = ~n;
		while(n>0){
			if((n&1)) e++;
			n>>=1;
		}
		return(e);
	},
	/* method: num2Prec()
	Returns rounded number with specified precision. Unlike .toFixed() or .toPrecision() trailing zeros are capped.
	Parameters:
		x - Number. To be evaluated.
		p - Number. Decimals.
	Returns:
		e - Number. The rounded number.
	*/
	num2Prec: function(x,p){
		if(!x) return 0;
		var n = Math.pow(10,p);
		return Math.round(x*n)/n;
	},
	/* method: pseudoRand()
	LCG (Linear congruential generator) pseudo-random number generator with salt (and pepper).
	Can be used to generate deterministic unique values, e.g. to decide if a specific entity should be spawned.
	Parameters:
		salt - Number. This is used to generate a pseudo random number.
		mode - Boolean. Optional. If specified and true the returned value is integer (Math.floor(n*100)).
	Returns:
		n - Number. If no mode specified in range [0...1] otherwise [0...100].
	Author: Jens Ayrton (Ahruman).
	*/
	pseudoRand: function(salt,mode){
		var n = system.scrambledPseudoRandomNumber(salt);
		if(mode) n = Math.floor(n*100);
		return(n);
	},
	/* method: rand()
	Random number in range 0...n.
	Parameters:
		n - Number. Maximum value.
	Returns:
		n - Number. Returns the random Number.
	Author: Dr.J R Stockton.
	*/
	rand: function(n){
		return(n*(Math.random()%1) | 0);
	},
	/* method: randSpan()
	Random number in range minN...maxN.
	Paramaters:
		minN - Number. Minimum.
		maxN - Number. Maximum.
	Returns:
		n - Number. The newly generated random Number.
	Author: Dr.J R Stockton.
	*/
	randSpan: function(minN,maxN){
		return minN+this.rand(maxN-minN+1);
	},
	/* method: strAddAlignedText()
	Returns padded text for a single line.
	Parameters:
		text		- String.
		alignment	- String. Optional. Either L, C or R. Default L.
	Returns:
		str			- String. Padded text.
	Author: Thargoid */
	strAddAlignedText: function(text,alignment){
		if(!text) return("");
		var validAlignment = "LCR",textWidth,padCount,padString;
		if(!alignment || validAlignment.indexOf(alignment)===-1) alignment = "L";
		alignment = alignment.toUpperCase();
		textWidth = defaultFont.measureString(text);
		if(textWidth>32 || alignment==="L") return(text);
		padCount = Math.floor((32-textWidth)/this.spacerWidth);
		if(alignment==="C") padCount = Math.floor(padCount/2);
		padString = this.strCreatePadString(padCount);
		return(padString+text);
	},
	/* method: strAddEdgeText()
	Returns two edge-aligned columns, one on the left and the other on the right for a single line.
	Parameters:
		leftText	- String.
		rightText	- String.
	Returns:
		str			- String. Aligned text.
	Author: Thargoid */
	strAddEdgeText: function(leftText,rightText){
		if(!rightText){
			if(leftText) return(leftText);
		}
		var textWidth = defaultFont.measureString(leftText)+defaultFont.measureString(rightText);
		if(textWidth>32){
			log("CCL strAdEdgeText","Text "+leftText+" plus "+rightText+" is too long for a single screen line!");
			return("");
		}
		var padCount = Math.floor((32-textWidth)/this.spacerWidth);
		var padString = this.strCreatePadString(padCount);
		return(leftText+padString+rightText);
	},
	/* method: strAddIndentedText()
	Returns indented line of text by the given distance (in ems) from the left.
	Parameters:
		text		- String.
		indent		- Number.
	Returns:
		str			- String. Indented text.
	Author: Thargoid */
	strAddIndentedText: function(text,indent){
		if(!text) return("");
		var textWidth = defaultFont.measureString(text);
		if(textWidth>32 || typeof(indent)!=='number' || indent<0 || indent>31.8) return(text);
		var padCount = Math.floor(indent/this.spacerWidth);
		var padString = this.strCreatePadString(padCount);
		return(padString+text);
	},
	/* method: strAdd2Columns()
	Returns line of text with two columns indented from the left margin by the given amount (in ems, 0-32).
	Parameters:
		leftText	- String.
		leftIndent	- Number.
		rightText	- String.
		rightIndent	- Number.
	Returns:
		str			- String. Text with 2 columns.
	Author: Thargoid */
	strAdd2Columns: function(leftText,leftIndent,rightText,rightIndent){
		if(!leftText || !rightText || typeof(leftIndent)!=='number' || typeof(rightIndent)!=='number' || leftIndent>rightIndent || leftIndent<0 || leftIndent>31.8 || rightIndent<0 || rightIndent>31.8) return("");
		var textWidth = defaultFont.measureString(leftText)+defaultFont.measureString(rightText);
		if(textWidth>32){
			log("CCL strAdd2Columns","2 column text is too long for a single screen line!");
			return("");
		}
		var leftPad = this.strCreatePadString(Math.floor(leftIndent/this.spacerWidth));
		var rightCount = rightIndent-(leftIndent+defaultFont.measureString(rightText));
		var rightPad;
		if(rightCount<=0){
			log("CCL strAdd2Columns","Left column overrun!");
			rightPad = "";
		} else rightPad = this.strCreatePadString(Math.floor(rightCount/this.spacerWidth));
		return(leftPad+leftText+rightPad+rightText);
	},
	/* method: strAdd3Columns()
	Returns text with three columns, left, centre and right.
	Parameters:
		leftText	- String.
		leftIndent	- Number.
		centreText	- String.
		centreIndent- Number.
		rightText	- String.
		rightIndent	- Number.
	Returns:
		str			- String. Text with 3 columns.
	Author: Thargoid */
	strAdd3Columns: function(leftText,leftIndent,centreText,centreIndent,rightText,rightIndent){
		if(!leftText || !centreText || !rightText || typeof(leftIndent)!=='number' || typeof(centreIndent)!=='number' || typeof(rightIndent)!=='number' || leftIndent>centreIndent || centreIndent>rightIndent || leftIndent<0 || leftIndent>32 || centreIndent<0 || centreIndent>32 || rightIndent<0 || rightIndent>32) return("");
		var textWidth = defaultFont.measureString(leftText)+defaultFont.measureString(centreText)+defaultFont.measureString(rightText);
		if(textWidth>32){
			log("CCL atrAdd3Columns","3 column text is too long for a single screen line!");
			return("");
		}
		var leftPad = this.strCreatePadString(Math.floor(leftIndent/this.spacerWidth));
		var centreCount = centreIndent-(leftIndent+defaultFont.measureString(centreText));
		var centrePad,rightPad;
		if(centreCount<=0){
			log("CCL strAdd3Columns","Left column overrun!");
			centrePad = "";
		} else centrePad = this.strCreatePadString(Math.floor(centreCount/this.spacerWidth));
		var rightCount = rightIndent-(centreIndent+defaultFont.measureString(rightText));
		if(rightCount<=0){
			log("CCL strAdd3Columns","Centre column overrun!");
			rightPad = "";
		} else rightPad = this.strCreatePadString(Math.floor(rightCount/this.spacerWidth));
		return(leftPad+leftText+centrePad+centreText+rightPad+rightText);
	},
	/* method: strCompareVersion()
	Compares version strings (like "2.0.3") against each other and returns number.
	It also strips pre/suffixes like "Build" or "Beta".
	Parameters:
		strA - String. Actual version.
		strB - String. Version to be compared against.
	Returns:
		0 - Number. If strA < strB
		1 - Number. If strA = strB
		2 - Number. If strA > strB
	*/
	strCompareVersion: function(strA,strB){
		var x = strA.split('.'),y = strB.split('.');
		if(isNaN(parseInt(x[0],null))){
			var n = x[0].search(/\d+/);
			if(n!==-1) x[0] = parseFloat(x[0].substr(n),null);
			else x.shift();
		}
		var xl = x.length,yl = y.length;
		if(!xl) return(0);
		for(var i=0;i<yl;i++){
			if(xl<=i) return(0);
			if(parseInt(x[i],null)>y[i]) return(2);
			if(parseInt(x[i],null)<y[i]) return(0);
		}
		return(1);
	},
	/* Internal method
	Author: Thargoid */
	strCreatePadString: function(count){
		var padString = "";
		if(!count || count<=0) return(padString);
		var counter = 0;
		for(counter=0;counter<count;counter++){
			padString = padString+this.spacer;
		}
		return(padString);
	},
	/* method: strEncrypt()
	Encrypts string.
	Parameters:
		str - String. To be encrypted. Minimum length 6 chars.
		pwd - String. Password. Minimum length 4 chars.
	Returns:
		enc_str - String/Boolean. Encrypted string or false.
	Author: Terry Yuen.
	*/
	strEncrypt: function(str,pwd){
		if(!str || !pwd || str.length<6 || pwd.length<4){log("Cabal_Common","Parameters error in encrypt."); return(false);}
		var prand = "";
		for(var i=0;i<pwd.length;i++) prand += pwd.charCodeAt(i).toString();
		var sPos = Math.floor(prand.length/5);
		var mult = parseInt(prand.charAt(sPos)+prand.charAt(sPos*2)+prand.charAt(sPos*3)+prand.charAt(sPos*4)+prand.charAt(sPos*5),null);
		var incr = Math.ceil(pwd.length/2);
		var modu = Math.pow(2,31)-1;
		if(mult<2){log("Cabal_Common","Algorithm cannot find a suitable hash."); return(false);}
		var salt = Math.round(Math.random()*1000000000)%100000000;
		prand += salt;
		while(prand.length>10) prand = (parseInt(prand.substring(0,10),null)+parseInt(prand.substring(10,prand.length),null)).toString();
		prand = (mult*prand+incr)%modu;
		var enc_chr = "",enc_str = "";
		for(var i=0;i<str.length;i++){
			enc_chr = parseInt(str.charCodeAt(i)^Math.floor((prand/modu)*255),null);
			if(enc_chr<16) enc_str += "0"+enc_chr.toString(16);
			else enc_str += enc_chr.toString(16);
			prand = (mult*prand+incr)%modu;
		}
		salt = salt.toString(16);
		while(salt.length<8) salt = "0"+salt;
		enc_str += salt;
		return(enc_str);
	},
	/* method: strDecrypt()
	Decrypts string.
	Parameters:
		str - String. To be decrypted. Minimum length 6 chars.
		pwd - String. Password. Minimum length 4 chars.
	Returns:
		dec_str - String/Boolean. Decrypted string or false.
	Author: Terry Yuen.
	*/
	strDecrypt: function(str,pwd){
		if(!str || !pwd || str.length<6 || pwd.length<4){log("Cabal_Common","Parameters error."); return(false);}
		var prand = "";
		for(var i=0;i<pwd.length;i++) prand += pwd.charCodeAt(i).toString();
		var sPos = Math.floor(prand.length/5);
		var mult = parseInt(prand.charAt(sPos)+prand.charAt(sPos*2)+prand.charAt(sPos*3)+prand.charAt(sPos*4)+prand.charAt(sPos*5),null);
		var incr = Math.round(pwd.length/2);
		var modu = Math.pow(2,31)-1;
		var salt = parseInt(str.substring(str.length-8,str.length),16);
		str = str.substring(0,str.length-8);
		prand += salt;
		while(prand.length>10) prand = (parseInt(prand.substring(0,10),null)+parseInt(prand.substring(10,prand.length),null)).toString();
		prand = (mult*prand+incr)%modu;
		var enc_chr = "",dec_str = "";
		for(var i=0;i<str.length;i+=2){
			enc_chr = parseInt(parseInt(str.substring(i,i+2),16)^Math.floor((prand/modu)*255),null);
			dec_str += String.fromCharCode(enc_chr);
			prand = (mult*prand+incr)%modu;
		}
		return(dec_str);
	},
	/* method: strGetCRC()
	Simple CRC from string. Does not create unique checksums. It uses Unicodes, but limits every char via &0xff and the output via &0x3fff.
	Parameters:
		str - String. Which should be used to create a CRC.
	Returns:
		crc - Number. The CRC.
	*/
	strGetCRC: function(str){
		var crc = 0, i = str.length;
		while(i--) crc += (str.charCodeAt(i)&0xff);
		return((crc&0x3fff));
	},
	/* method: strDateFromMinutes()
	Takes an integer number of minutes and converts it into a string.
	The returned string has the form 'd' days, 'h' hours and 'm' minutes, accomodating all cases of singular, plural, and 0.
	Parameters:
		minutes - Number. The time in minutes.
	Returns:
		timeString - String.
	Author: Commander McLane.
	*/
	strDateFromMinutes: function(minutes){
		var ago = false;
		if(minutes<0) ago = true;
		minutes = Math.abs(minutes);
		var daysComponent = Math.floor(minutes/24/60);
		var hoursComponent = Math.floor((minutes%(24*60))/60);
		var minutesComponent = minutes%60;
		var timeString = "";
		if(daysComponent>0) timeString += daysComponent + " day";
		if(daysComponent>1) timeString += "s";
		if(timeString!==""){
			if(hoursComponent>0 && minutesComponent>0) timeString += ", ";
			else if(hoursComponent>0) timeString += " and ";
		}
		if(hoursComponent>0) timeString += hoursComponent + " hour";
		if(hoursComponent>1) timeString += "s";
		if(timeString!=="" && minutesComponent>0) timeString += " and ";
		if(minutesComponent>0) timeString += minutesComponent + " minute";
		if(minutesComponent>1) timeString += "s";
		if(timeString==="") timeString = "no time";
		if(ago) timeString += " ago";
		return timeString;
	},
	/* method: strDateRelative()
	Returns string (e.g. '4 minutes ago').
	Parameters:
		inSec - Number. In seconds.
	Returns:
		str - String.
	*/
	strDateRelative: function(inSec){
		var reference = clock.absoluteSeconds,delta,formats,format,i,len;
		var sec = 1; var min = 60*sec; var hour = 60*min; var day = 24*hour; var week = 7*day; var year = day*365; var month = year/12;
		if(inSec<=0){
			delta = reference-(clock.absoluteSeconds+inSec);
			formats = [[0.7*min,'just now'],[1.5*min,'a minute ago'],[60*min,'minutes ago',min],[1.5*hour,'an hour ago'],
				[0.7*day,'hours ago',hour],[day,'today'],[2*day,'yesterday'],[7*day,'days ago',day],[1.5*week,'a week ago'],
				[month,'weeks ago',week],[1.5*month,'a month ago'],[year,'months ago',month],[1.5*year,'a year ago'],
				[Number.MAX_VALUE,'years ago',year]];
		} else {
			delta = (clock.absoluteSeconds+inSec)-reference;
			formats = [[0.7*min,'just now'],[1.5*min,'a minute'],[60*min,'minutes',min],[1.5*hour,'an hour'],
				[0.7*day,'hours',hour],[day,'today'],[2*day,'tomorrow'],[7*day,'days',day],[1.5*week,'a week'],
				[month,'weeks',week],[1.5*month,'a month'],[year,'months',month],[1.5*year,'a year'],
				[Number.MAX_VALUE,'years',year]];
		}
		for(i=-1,len=formats.length;++i<len;){
			format = formats[i];
			if(delta<format[0]) return format[2] == undefined?format[1]:Math.round(delta/format[2])+' '+format[1];
		}
	},
	/* method: strLZ()
	Returns string with a leading zero if n<10.
	Parameters:
		n - Number. To be evaluated.
	Returns:
		str - String. With leading zero if necessary.
	Author: Dr.J R Stockton.
	*/
	strLZ: function(n){
		return (n!=null&&n<10&&n>=0?"0":"")+n;
	},
	/* method: strLZZ()
	Returns string with a leading zeros if n<100. Uses <strLZ()> as subfunction.
	Parameters:
		n - Number. To be evaluated.
	Returns:
		str - String. With leading zeros if necessary.
	Author: Dr.J R Stockton.
	*/
	strLZZ: function(n){
		return(n!=null&&n<100&&n>=0?"0"+this.strLZ(n):""+n);
	},
	/* method: strNLZ()
	Returns string with length of len filled with leading zeros.
	Parameters:
		n - Number. To be evaluated.
		len - Number. Length. Maximum is 10.
	Returns:
		str - String. Filled up with leading zeros.
	*/
	strNLZ: function(n,len){
		var s = n.toString();
		if(!len || len<s.length) len = s.length;
		if(len && len>10) len = 10;
		if(s.length<len) s = ("0000000000"+s).slice(-len);
		return s;
	},
	/* method: strRandom()
	Returns random string with length of l.
	Parameters:
		l - Number. Length.
	Returns:
		str - String.
	*/
	strRandom: function(l){
		var str = "";
		for(var j=0;j<l;j++) str += String.fromCharCode(97+this.rand(26));
		return str;
	},
	/* method: strRandomInt()
	Returns random numbered string with length of l.
	Parameters:
		l - Number. Length in range of 1...9.
	Returns:
		str - String.
	*/
	strRandomInt: function(l){
		var str = String(1000000000+Math.random()*1000000000 | 0);
		return str.substr(0,l);
	},
	/* method: strScreenSubString()
	Formats the string str into lines of at most chrs characters in length.
	Parameters:
		str - String. To be used.
		chrs - Number. The length.
	Returns:
		out - String. With specified length.
	*/
	strScreenSubString: function(str,chrs){
		var out = "";
		for(var d=0;d<(str.length/chrs);d++) out += str.substr(d*chrs,chrs)+"\n";
		return(out);
	},
	/* method: strScreenString()
	Splits string in pieces of chrs, with intermediate linebreaks for missionscreens and fills up current line.
	If splitting is not possible or a word is longer than chrs, <strScreenSubString()> will be used.
	Parameters:
		str - String. To be used.
		mode - Boolean. Optional. If mode is false or not specified it replaces special signs like linebreaks with spaces.
		chrs - Number. Optional. Length. If not specified default of 70 is used.
	Returns:
		display - String. With linebreaks.
	*/
	strScreenString: function(str,mode,chrs){
		if(!mode) str = str.replace(/[\t\r\f\n\v]/g," ");
		if(!chrs) chrs = 70;
		var display = "", splob = str.split(" ");
		var l = splob.length, temp = "";
		if(l<2) display = this.strScreenSubString(str,chrs);
		else {
			for(var o=0;o<l;o++){
				if(splob[o].length>chrs){
					display += temp+"\n";
					display += this.strScreenSubString(splob[o],chrs);
					temp = "";
					continue;
				}
				if(!o && splob[o].length<chrs){
					temp = splob[o];
					continue;
				}
				if((temp.length+splob[o].length)<chrs){
					if(temp!=="") temp += " "+splob[o];
					else temp += splob[o];
				} else {
					display += temp+"\n";
					temp = splob[o];
				}
			}
			if(temp!=="") display += temp;
		}
		return(display);
	},
	/* method: strSplitToPhrases()
	Splits string into n words phrases.
	Parameters:
		str - String. To be used.
		n - Number. Words.
	Returns:
		out - Array. Containing Strings with n words per element.
	*/
	strSplitToPhrases: function(str,n){
		var arr = str.split(" ");
		var l = arr.length, out = [], sub = "";
		for(var o=0;o<l;o++){
			if(!arr[o] || arr[o]==="") continue;
			if(o>l-n) break;
			sub = "";
			for(var i=o;i<Math.min(o+n,l);i++){
				if(!arr[i] || arr[i]=="") continue;
				sub += arr[i]+" ";
			}
			sub = sub.substr(0,sub.length-1);
			out.push(sub);
		}
		return(out);
	},
	/* method: strToWidth()
	Returns string with specific length based on Oolites font size.
	If stringwidth > max it is truncated, otherwise filled up with space or specified chr.
	Parameters:
		str - String. To be used.
		max - Number. Maximum width.
		chr - String. Char to be used to fill up if stringwidth < max.
	Returns:
		str - String. With specified width.
	*/
	strToWidth: function(str,max,chr){
		var l = defaultFont.measureString(str),c,d;
		if(chr) c = defaultFont.measureString(chr);
		if(l>max){
			while(l>max){
				str = str.substr(0,str.length-1);
				d = defaultFont.measureString(str[str.length-1]);
				l -= d;
			}
		} else {
			while(l<max){
				if(chr && chr!==" "){
					str += chr;
					l += c;
				} else {
					if(max-l>1.7){
						str += this.spacerStandard;
						l += this.spacerWidthStandard;
					} else {
						if(max-l>0.1){
							str += this.spacer;
							l += this.spacerWidth;
						} else break;
					}
				}
			}
		}
		return(str);
	},
	/* method: strTrim()
	Returns trimmed string.
	Parameters:
		str  - String.
	Returns:
		str - String.
	*/
	strTrim: function(str){
		return(str.replace(/[\b\f\r\n]/g,"").toString().replace(/\s+$|^\s+/g,""));
	},
	/* method: arrDiff()
	Returns difference between two arrays.
	Parameters:
		ar1 - Array. First.
		ar2 - Array. Second.
	Returns:
		diff - Array. Difference between two arrays.
	*/
	arrDiff: function(ar1,ar2){
		var a=[], diff=[];
		for(var i=0;i<ar1.length;i++) a[ar1[i]]=true;
		for(var i=0;i<ar2.length;i++){
			if(a[ar2[i]]) delete a[ar2[i]];
			else a[ar2[i]]=true;
		}
		return diff;
	},
	/* method: arrRemoveByValue()
	Removes specific entry from array.
	Parameters:
		arr - Array. From which things should be removed.
		val - Object. Value that has to be removed.
	Returns:
		arr - Array. The cleaned array.
	*/
	arrRemoveByValue: function(arr,val){
		for(var i=0;i<arr.length;i++){
			if(arr[i]===val){
				arr.splice(i,1);
				break;
			}
		}
		return(arr);
	},
	/* method: arrShuffle()
	Returns shuffled array.
	Parameters:
		arr - Array.
	Returns:
		SHU - Array.
	Author: Dr.J R Stockton.
	*/
	arrShuffle: function(arr){
		var J,K,T,SHU=[].concat(arr);
		for(J=SHU.length-1;J>0;J--){
			K = this.rand(J+1);
			T = SHU[J];
			SHU[J] = SHU[K];
			SHU[K] = T;
		}
		return SHU;
	},
	/* method: arrSortByProperty()
	Sort array containing objects by specific property.
	Parameters:
		arr - Array. To be sorted.
		prop - String. Propertyname.
	Returns:
		what - Array. Sorted by specified property.
	*/
	arrSortByProperty: function(arr,prop){
		function sortByPropName(a,b){
			var x = a[prop];
			var y = b[prop];
			return((x<y)?-1:((x>y)?1:0));
		}
		arr.sort(sortByPropName);
		return(arr);
	},
	/* method: arrUnique()
	Removes duplicates from array.
	Parameters:
		arr - Array. The input array, from which duplicates are to be removed.
	Returns:
		r - Array. The cleaned array.
	*/
	arrUnique: function(arr){
		var r = [];
		o:for(var i=0,n=arr.length;i<n;i++){
			for(var x=0,y=r.length;x<y;x++) if(r[x]==arr[i]) continue o;
			r[r.length] = arr[i];
		}
		return r;
	},
	/* method: mapCoordsDirection()
	Returns the general direction of the target coordinates compared to the current (or specified) system coordinates.
	Parameters:
		posA - Vector/ID. Map coordinates in LYs or simply a system.ID.
		posB - Vector/ID. Optional. Map coordinates in LYs or simply a system.ID. Defaults to current system coordinates.
	Returns:
		str - String.
	Author: Eric Walch.
	*/
	mapCoordsDirection: function(posA,posB){
		if(typeof(posA)==='undefined' || (typeof(posA)==='object' && !posA)) return("current system");
		var pos1,pos2,dx,dy,x2,y2;
		if(typeof(posA)==='number') pos2 = System.infoForSystem(galaxyNumber,posA).coordinates;
		else pos2 = new Vector3D(posA);
		if(typeof(posB)==='number') pos1 = System.infoForSystem(galaxyNumber,posB).coordinates;
		else if(!posB) pos1 = system.info.coordinates;
		else pos1 = new Vector3D(posB);
		if(pos1.distanceTo(pos2)<0.00001) return("current system");
		dx = pos2.x-pos1.x;
		dy = pos2.y-pos1.y;
		x2 = Math.abs(dx);
		y2 = Math.abs(dy);
		if(x2>2.41421*y2) return(dx>0?"east":"west");
		if(y2>1.732*x2) return(dy<0?"north":"south");
		if(dx>0) return(dy<0?"north-east":"south-east");
		return(dy<0?"north-west":"south-west");
	},
	/* method: oxpVersionTest()
	Checks required OXPs and versions. The array is expected to contain two elements for each script and version must be null for legacy scripts!
	Parameters:
		who - String. Actual scriptname. It's only used to log for which OXP the requirements are not fullfilled.
		requires - Array. Holds for every required OXP the scriptname and version. To check only for existance set version to null.
		quiet - Boolean. Optional. If true don't log, only check.
	Returns:
		checked - Boolean.
		- false - If a requirement is not fullfilled or OXP is deactivated,
		- true - Otherwise.
	Note:
	The method checks for a property 'deactivated' in OXPs to determine if they are deactived.
	*/
	oxpVersionTest: function(who,requires,quiet){
		var checked = true;
		for(var i=0;i<requires.length;i+=2){
			if(typeof(worldScripts[requires[i]])==='undefined'){
				if(!requires[i+1]){checked = false; if(!quiet) log("Check",who+" requirement not matched : "+requires[i]+" not installed.");}
				else {checked = false; if(!quiet) log("Check",who+" requirement not matched : "+requires[i]+" min. version "+requires[i+1]+" not installed.");}
				continue;
			} else {
				if(requires[i+1] && !this.strCompareVersion(worldScripts[requires[i]].version,requires[i+1])){checked = false;
					if(!quiet) log("Check",who+" requirement not matched : "+requires[i]+" required min. version "+requires[i+1]+" not found.");
					continue;
				}
			}
			if(worldScripts[requires[i]].deactivated) checked = false;
		}
		return(checked);
	},
	/* method: oxpVersionTest2Array()
	Checks required OXPs and versions. The array is expected to contain two elements for each script and version must be null for legacy scripts!
	Parameters:
		who - String. Actual scriptname. It's only used to log for which OXP the requirements are not fullfilled.
		requires - Array. Holds for every required OXP the scriptname and version. To check only for existance set version to null.
		quiet - Boolean. Optional. If true don't log, only check.
	Returns:
		checked - Array. With values for checked elements of requires.
		- 2 - version is greater,
		- 1 - version is equal,
		- 0 - not installed,
		- -1 - OXP is deactivated,
		- -2 - version is lower.
	*/
	oxpVersionTest2Array: function(who,requires,quiet){
		var checked = [],check = 0,loop = requires.length;
		for(var i=0;i<loop;i+=2){
			if(typeof(worldScripts[requires[i]])==='undefined'){
				checked.push(0);
				if(!requires[i+1]){if(!quiet) log("Check",who+" requirement not matched : "+requires[i]+" not installed.");}
				else {if(!quiet) log("Check",who+" requirement not matched : "+requires[i]+" min. version "+requires[i+1]+" not installed.");}
				continue;
			} else {
				if(worldScripts[requires[i]].deactivated) checked.push(-1);
				else if(requires[i+1]){
					check = this.strCompareVersion(worldScripts[requires[i]].version,requires[i+1]);
					if(!check){
						checked.push(-2);
						if(!quiet) log("Check",who+" requirement not matched : "+requires[i]+" required min. version "+requires[i+1]+" not found.");
						continue;
					} else checked.push(check);
				}
			}
		}
		return(checked);
	},
	/* method: entModelView()
	Spawns or resets the modelview entity and returns it.
	Parameters:
		which - String. Optional. Role to be used for modelview. If not specified defaults to 'cabal_common_modelview'.
	Returns:
		mv - Entity. Returns modelview Entity or null.
	*/
	entModelView: function(which){
		if(!player.ship.docked) return(false);
		var mv = null;
		if(!which) which = 'cabal_common_modelview';
		if(!system.countShipsWithRole(which)) mv = system.addShips(which,1,player.ship.position,5000.0)[0];
		else mv = system.shipsWithPrimaryRole(which)[0];
		if(mv) mv.AIState = 'GLOBAL';
		return(mv);
	},
	/* method: entFaceEntity()
	This function causes ent1 to be reorientated such that it faces ent2. A common use-case is when spawning stations whose docking
	tunnel should face another entity, typically the main planet. Reorienting is instantly.
	Parameters:
		ent1 - Entity. To be reoriented.
		ent2 - Entity. To which ent1 should be oriented.
	Returns:
		nothing.
	*/
	entFaceEntity: function(ent1,ent2){
		var targetVector = ent2.position.subtract(ent1.position).direction();
		var angle = ent1.heading.angleTo(targetVector);
		var cross = ent1.heading.cross(targetVector).direction();
		ent1.orientation = ent1.orientation.rotate(cross,-angle);
		return;
	},
	/* method: entScreenCornerPos()
	Missionscreen model corner positioning.
	Parameters:
		level - Number. Works best with values between 0 and 0.35.
		signx - Number. Used as sign mantissa (-1...1) for X axis.
		signy - Number. Used as sign mantissa (-1...1) for Y axis.
		further - Number. Multiplier for Z axis. Defaults to 1.3.
	Returns:
		nothing.
	*/
	entScreenCornerPos: function(level,signx,signy,further){
		var pos = mission.displayModel.position,w = 1024/oolite.gameSettings.gameWindow.width,h = 768/oolite.gameSettings.gameWindow.height;
		pos.x = level*pos.z*signx;
		pos.y = level*pos.z*0.75*signy;
		if(further) pos.z *= further;
		else pos.z *= 1.3;
		var wh = w/h;
		pos = pos.add([w*wh-1,h*wh-1,0]);
		mission.displayModel.position=pos;
		return;
	},
	/* method: screenGCD()
	Returns greatest common denominator. Subfunction.
	Parameters:
		a - Number.
		b - Number.
	Returns:
		n - Number.
	*/
	screenGCD: function(a,b){
		return((b==0)?a:this.screenGCD(b,a%b));
	},
	/* method: screenAspect()
	Returns aspect ratio. Subfunction.
	Parameters:
		w - Number. Width.
		h - Number. Height.
		d - Number. Denominator.
	Returns:
		n - Number.
	*/
	screenAspect: function(w,h,d){
		if(!d) d = this.screenGCD(w,h);
		return((w/d)/(h/d));
	},
	/* method: screenChecks()
	Returns array with width,height,denominator and aspect ratio.
	Parameters:
		w - Number. Width. Optional.
		h - Number. Height. Optional.
	Returns:
		a - Array.
	*/
	screenChecks: function(w,h){
		if(!w) w = oolite.gameSettings.gameWindow.width;
		if(!h) h = oolite.gameSettings.gameWindow.height;
		var d = this.screenGCD(w,h);
		var a = this.screenAspect(w,h,d);
		return([w,h,d,a]);
	}
};

/* class: Cabal_Common_BinSearch
This is the main class for the binary search tree with its members. This search tree uses two corresponding entries.
*/
this.Cabal_Common_BinSearch = function(){this._cclbinsearch = null;}
Cabal_Common_BinSearch.prototype = {
	constructor: Cabal_Common_BinSearch,
	/* property: internalVersion
	Property to give OXPs a chance to check the required lib min. version easily.
	*/
	internalVersion: 15,
	/* method: add()
	Adds nodes to the search tree with the corresponding entry.
	Parameters:
		key - String. Add node with key.
		value - Object. Corresponding entry
	Returns:
		nothing
	*/
	add: function(key,value){
		var node = {key: key,left: null,right: null, value: value},current;
		if(this._cclbinsearch === null) this._cclbinsearch = node;
		else {
			current = this._cclbinsearch;
			while(true){
				if(key < current.key){
					if(current.left === null){
						current.left = node;
						break;
					} else current = current.left;
				} else if(key > current.key){
					if(current.right === null){
						current.right = node;
						break;
					} else current = current.right;
				} else break;
			}
		}
		return;
	},
	/* method: contains()
	Returns corresponing entry or false.
	Parameters:
		key - String. Search for entry.
	Returns:
		value - Object. Corresponding entry or false.
	*/
	contains: function(key){
		var found = false,current = this._cclbinsearch;
		while(!found && current){
			if(key < current.key) current = current.left;
			else if(key > current.key) current = current.right;
			else found = true;
		}
		if(!found) return (false);
		return (current.value);
	},
	// Internal
	traverse: function(process){
		function inOrder(node){
			if(node){
				if(node.left !== null) inOrder(node.left);
				process.call(this, node);
				if(node.right !== null) inOrder(node.right);
			}
		}
		inOrder(this._cclbinsearch);
	},
	/* method: size()
	Returns the number of nodes in the search tree.
	Parameters:
		none
	Returns:
		length - Number. Number of nodes in searchtree.
	*/
	size: function(){
		var length = 0;
		this.traverse(function(node){length++;});
		return length;
	},
	/* method: toArray()
	Returns an array conaining all nodes in the search tree. The nodes are processed in-order.
	Parameters:
		none
	Returns:
		result - Array. Entries in search tree.
	*/
	toArray: function(){
		var result = [];
		this.traverse(function(node){result.push(node.key,node.value);});
		return result;
	},
	/* method: toString()
	Returns a comma-separated string consisting of all entries. The nodes are processed in-order.
	Parameters:
		separator - String. Optional. If specified separator is used to separate the elements instead of a commata.
	Returns:
		result - String. Concatenation of all entries in search tree.
	*/
	toString: function(separator){
		if(separator) return this.toArray().join(separator);
		else return this.toArray().toString();
	}
};

/* class: Cabal_Common_2DCollision
This is the main class for the 2D checks with its members.
*/
this.Cabal_Common_2DCollision = function(){}
Cabal_Common_2DCollision.prototype = {
	constructor: Cabal_Common_2DCollision,
	/* property: internalVersion
	Property to give OXPs a chance to check the required lib min. version easily.
	*/
	internalVersion: 15,
	/* method: boundingBox()
	Checks if point is inside the bounding box.
	Parameters:
		minmax - Array. 4 positions in a.x,a.y,b.x,b.y
		pos - Vector. Position to be checked.
	Returns:
		inside - Boolean. True if inside.
	*/
	boundingBox: function(minmax,pos){
		if(pos.x<minmax[0] || pos.x>minmax[1] || pos.y<minmax[2] || pos.y>minmax[3]) return(false);
		return(true);
	},
	/* method: pointInPoly()
	Checks if a point is inside a polygon.
	Parameters:
		nvert - Number of vertices in the polygon.
		vertx, verty - Arrays containing the x- and y-coordinates of the polygon's vertices.
		pos - Vector. x- and y-coordinate of the test point.
		con - Boolean. Concave shapes may need to set it.
	Return:
		inside - Boolean. True if inside.
	*/
	pointInPoly: function(nvert,vertx,verty,pos,con){
		var i,j,c = false,r1=0;
		for(i=0,j=nvert-1;i<nvert;j=i++){
			if(((verty[i]>pos.y)!==(verty[j]>pos.y)) && (pos.x<(vertx[j]-vertx[i])*(pos.y-verty[i])/(verty[j]-verty[i])+vertx[i])){
				c = !c;
				r1++;
			}
			if(!con && r1>1) break;
		}
		return(c);
	},
	/* method: pointOnLine()
	Checks if point is on a specified line with a tolerance of 0.4.
	This method has a problem with vertical lines. If it needs to be more accurate use .pointOnLineB()
	Parameters:
		pa - Vector. Starting point of the line.
		pb - Vector. End point of the line.
		pos - Vector. Position to be checked.
	Return:
		online - Boolean. True if on line (within tolerance).
	*/
	pointOnLine: function(pa,pb,pos){
		var f = function(somex){return(pb.y-pa.y)/(pb.x-pa.x)*(somex-pa.x)+pa.y;};
		if((pa.x>pos.x+0.4 && pb.x>pos.x+0.4) || (pa.x<pos.x-0.4 && pb.x<pos.x-0.4)) return(false);
		if(pb.x<pa.x){var t = pa; pa = pb; pb = t;} //Switch
		if(pa.x>pb.x+0.4 || pa.x<pb.x-0.4) return(Math.abs(f(pos.x)-pos.y)<0.4 && pos.x>=pa.x && pos.x<=pb.x);
		return(((pos.y>=pa.y && pos.y<=pb.y) || (pos.y<=pa.y && pos.y>=pb.y)) && (pos.x>=pa.x && pos.x<=pb.x) || (pos.x<=pa.x && pos.x>=pb.x));
	},
	/* method: pointOnLineB()
	Checks if point is on a specified line.
	Parameters:
		p1 - Vector. Starting point of the line.
		p2 - Vector. End point of the line.
		p3 - Vector. Position to be checked.
		tol - Number. Tolerance. Default 0.01.
	Return:
		online - Boolean. True if on line (within tolerance).
	Author: Paul Bourke
	*/
	pointOnLineB: function(p1,p2,p3,tol){
		var s,rnum,denom,check,distanceLine,distanceSegment,dist1,dist2;
		if(typeof(tol)!=='number') tol = 0.01;
		rnum = (p3.x - p1.x) * (p2.x - p1.x) + (p3.y - p1.y) * (p2.y - p1.y);
		denom = (p2.x - p1.x) * (p2.x - p1.x) + (p2.y - p1.y) * (p2.y - p1.y);
		check = rnum / denom;
		s = ((p1.y-p3.y) * (p2.x - p1.x) - (p1.x - p3.x) * (p2.y - p1.y) ) / denom;
		distanceLine = Math.abs(s) * Math.sqrt(denom);
		if(check >= 0 && check <= 1) distanceSegment = distanceLine;
		else {
			dist1 = (p3.x - p1.x) * (p3.x - p1.x) + (p3.y - p1.y) * (p3.y - p1.y);
			dist2 = (p3.x - p2.x) * (p3.x - p2.x) + (p3.y - p2.y) * (p3.y - p2.y);
			if(dist1 < dist2) distanceSegment = Math.sqrt(dist1);
			else distanceSegment = Math.sqrt(dist2);
		}
		return((distanceSegment<tol));
	}
};

// Functionality to check systems and get more infos about the session
this.moreInfo = {
	jumpCount: 0,
	lastJumpTime: clock.absoluteSeconds,
	lastJumpDiff: 0,
	avgEnts: system.allShips.length,
	conEnts: [system.allShips.length],
	avgEntsOXP: system.allShips.length,
	conEntsOXP: [system.allShips.length],
	galaxyStats : null,
	connected : null
};
this.startUp = function()
{
	delete this.startUp;
	this.getGalaxy(); // planetinfo.plist is already processed
	this.consTimer = new Timer(this,this.checkConnection,0.1);
};
this.playerEnteredNewGalaxy = function()
{
	this.getGalaxy();
	this.consTimer = new Timer(this,this.checkConnection,0.1);
};
this.shipWillEnterWitchspace = function()
{
	if(this.delayedTimer){this.delayedTimer.stop(); delete this.delayedTimer;}
	this.moreInfo.lastJumpDiff = clock.absoluteSeconds-this.moreInfo.lastJumpTime;
	this.moreInfo.lastJumpTime = clock.absoluteSeconds;
	this.moreInfo.jumpCount++;
	this.updateGalaxyStats();
};
this.shipWillExitWitchspace = function()
{
	this.updateGalaxyStats();
};
this.shipExitedWitchspace = function()
{
	if(!system.isInterstellarSpace && this.moreInfo.jumpCount<30){
		this.moreInfo.conEnts.push(system.allShips.length);
		var n=0;
		for(var a=0;a<this.moreInfo.conEnts.length;a++) n+=this.moreInfo.conEnts[a];
		this.moreInfo.avgEnts = Math.ceil(n/this.moreInfo.conEnts.length);
		if(this.moreInfo.lastJumpDiff>10) this.delayedTimer = new Timer(this,this.delayedShipExitedWitchspace,10);
	}
};
this.delayedShipExitedWitchspace = function()
{
	if(this.moreInfo.jumpCount<30){
		this.moreInfo.conEntsOXP.push(system.allShips.length);
		var n=0;
		for(var a=0;a<this.moreInfo.conEntsOXP.length;a++) n+=this.moreInfo.conEntsOXP[a];
		this.moreInfo.avgEntsOXP = Math.ceil(n/this.moreInfo.conEntsOXP.length);
	}
	delete this.delayedTimer;
};
this.updateGalaxyStats = function()
{
	if(system.isInterstellarSpace) return;
	var inf = this.moreInfo.galaxyStats.contains(system.ID),what = ["coordinates","description","economy","government","inhabitant","inhabitants",["market","market"],"name","population","productivity","radius",["sun","sun_gone_nova"],"techlevel"];
	var l = what.length,temp;
	for(var i=0;i<l;i++){
		temp = what[i];
		if(typeof(temp)!=="string"){
			if(inf[temp[0]]===system.info[temp[1]]) continue;
			if(typeof(inf[temp[0]])!=="undefined") inf[temp[0]] = (system.info[temp[1]]?false:true);
		} else {
			if(inf[temp]===system.info[temp]) continue;
			inf[temp] = system.info[temp];
		}
	}
	return;
};
this.getGalaxy = function()
{
	this.moreInfo.galaxyStats = new this.Cabal_Common_BinSearch(); // Binary tree by ID
	var c,gn=galaxyNumber,sy,r;
	this.gInfo = [];
	// Accessing system properties adds >250 ms
	// except coordinates - this adds only ~4 ms
	for(var i=0;i<256;i++){
		c=System.infoForSystem(gn,i);
		sy = {
			coordinates: {
				x:c.coordinates.x,
				y:c.coordinates.y,
				z:c.coordinates.z
			},
			description: c.description,
			economy: c.economy,
			government: c.government,
			ID: i,
			inhabitant: c.inhabitant,
			inhabitants: c.inhabitants,
			market: c.market,
			name: c.name,
			population: c.population,
			productivity: c.productivity,
			radius: c.radius,
			sun: (c.sun_gone_nova?false:true),
			techlevel: c.techlevel
		};
		r = {
			x: c.coordinates.x,
			y: c.coordinates.y,
			id: i,
			done: 0
		};
		this.gInfo.push(r);
		this.moreInfo.galaxyStats.add(i,sy);
	}
	return;
};
// Find connections
this.checkConnection = function()
{
	var helper = new this.Cabal_Common();
	var m = 1.0214285714285714285714285714286;
	var check = [],cur = [],cpp = player.ship.galaxyCoordinatesInLY,dist,temp;
	var cp = {
		x: cpp.x,
		y: cpp.y,
		z: 0
	};
	this.gInfo = helper.arrSortByProperty(this.gInfo,"x");
	this.moreInfo.connected = [];
	// Start loop
	for(var i=0;i<256;i++){
		temp = this.gInfo[i];
		if(temp.x>cp.x+7.15) break;
		if(temp.x<cp.x-7.15 || temp.y<cp.y-7.15 || temp.y>cp.y+7.15) continue;
		dist = Math.sqrt(((cp.x-temp.x)*(cp.x-temp.x)*m)+(cp.y-temp.y)*(cp.y-temp.y));
		if(dist<7.341) cur.push(temp);
	}
	// Grow
	while(cur.length){
		check = check.concat(cur);
		cur = [];
		for(var o=0;o<check.length;o++){
			cp = check[o];
			if(cp.id===system.ID || cp.done) continue;
			for(var i=0;i<256;i++){
				temp = this.gInfo[i];
				if(temp.x>cp.x+7.15) break;
				if(temp.x<cp.x-7.15 || temp.y<cp.y-7.15 || temp.y>cp.y+7.15) continue;
				dist = Math.sqrt(((cp.x-temp.x)*(cp.x-temp.x)*m)+(cp.y-temp.y)*(cp.y-temp.y));
				if(dist<7.341 && check.indexOf(temp)===-1 && cur.indexOf(temp)===-1) cur.push(temp);
			}
		}
		for(var m=0;m<check.length;m++){
			temp = this.gInfo.indexOf(check[m]);
			this.gInfo[temp].done = 1;
		}
	}
	for(var t=0;t<check.length;t++) this.moreInfo.connected.push(check[t].id);
	if(this.consTimer) delete this.consTimer;
	delete this.gInfo;
	return;
};
